"""Top-level package for NetBox Napalm Plugin."""

__author__ = """Arthur Hanson"""
__email__ = "ahanson@netboxlabs.com"
__version__ = "0.1.5"


from extras.plugins import PluginConfig


class NapalmPlatformConfig(PluginConfig):
    name = "netbox_napalm_plugin"
    verbose_name = "NetBox Napalm Plugin"
    description = "NetBox plugin for Napalm."
    version = __version__
    author = __author__
    author_email = __email__
    base_url = "netbox_napalm_plugin"
    required_settings = ['NAPALM_USERNAME', 'NAPALM_PASSWORD', ]
    default_settings = {
        'NAPALM_TIMEOUT': 30,
        'NAPALM_ARGS': {},
    }
    min_version = '3.5.0-dev'
    max_version = '3.6.99'


config = NapalmPlatformConfig



from django import forms
from ipam.models import Prefix
from netbox.forms import NetBoxModelFilterSetForm, NetBoxModelForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField

from .models import NapalmPlatformConfig


class NapalmPlatformConfigForm(NetBoxModelForm):
    class Meta:
        model = NapalmPlatformConfig
        fields = (
            "platform",
            "napalm_driver",
            "napalm_args",
            "tags",
        )



from dcim.api.serializers import NestedPlatformSerializer
from netbox.api.serializers import NetBoxModelSerializer
from rest_framework import serializers

from netbox_napalm_plugin.models import NapalmPlatformConfig


class NapalmPlatformConfigSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:netbox_napalm_plugin-api:napalmplatformconfig-detail"
    )
    platform = NestedPlatformSerializer()

    class Meta:
        model = NapalmPlatformConfig
        fields = [
            "id",
            "url",
            "platform",
            "napalm_driver",
            "napalm_args",
            "tags",
            "created",
            "last_updated",
        ]


class DeviceNAPALMSerializer(serializers.Serializer):
    method = serializers.JSONField()



# Generated by Django 4.1.5 on 2023-02-15 16:57

import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("extras", "0084_staging"),
        ("dcim", "0167_module_status"),
    ]

    operations = [
        migrations.CreateModel(
            name="NapalmPlatformConfig",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("napalm_driver", models.CharField(blank=True, max_length=50)),
                ("napalm_args", models.JSONField(blank=True, null=True)),
                (
                    "platform",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="napalm",
                        to="dcim.platform",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
            ],
            options={
                "ordering": ("pk",),
            },
        ),
    ]



import django_filters
from dcim.models import Platform
from django.utils.translation import gettext as _
from netbox.filtersets import (NetBoxModelFilterSet,
                               OrganizationalModelFilterSet)

from .models import NapalmPlatformConfig


class NapalmPlatformConfigFilterSet(NetBoxModelFilterSet):
    platform_id = django_filters.ModelMultipleChoiceFilter(
        field_name="platform",
        queryset=Platform.objects.all(),
        label=_("Platform (ID)"),
    )
    platform = django_filters.ModelMultipleChoiceFilter(
        field_name="platform__slug",
        queryset=Platform.objects.all(),
        to_field_name="slug",
        label=_("Platform (slug)"),
    )

    class Meta:
        model = NapalmPlatformConfig
        fields = ["id", "napalm_driver"]



from django.urls import path
from netbox.views.generic import ObjectChangeLogView

from . import models, views

urlpatterns = (
    path("napalm/", views.NapalmPlatformConfigListView.as_view(), name="napalmplatformconfig_list"),
    path(
        "napalm/add/", views.NapalmPlatformConfigEditView.as_view(), name="napalmplatformconfig_add"
    ),
    path("napalm/<int:pk>/", views.NapalmPlatformConfigView.as_view(), name="napalmplatformconfig"),
    path(
        "napalm/<int:pk>/edit/",
        views.NapalmPlatformConfigEditView.as_view(),
        name="napalmplatformconfig_edit",
    ),
    path(
        "napalm/<int:pk>/delete/",
        views.NapalmPlatformConfigDeleteView.as_view(),
        name="napalmplatformconfig_delete",
    ),
    path(
        "napalm/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="napalmplatformconfig_changelog",
        kwargs={"model": models.NapalmPlatformConfig},
    ),
)



# api/urls.py
from netbox.api.routers import NetBoxRouter

from .views import NapalmPlatformConfigViewSet

router = NetBoxRouter()
router.register("napalmplatformconfig", NapalmPlatformConfigViewSet)
urlpatterns = router.urls



from dcim.models.devices import Platform
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _
from netbox.models import NetBoxModel


class NapalmPlatformConfig(NetBoxModel):
    platform = models.OneToOneField(
        Platform,
        on_delete=models.CASCADE,
        related_name="napalm",
    )
    napalm_driver = models.CharField(
        max_length=50,
        blank=True,
        verbose_name="NAPALM driver",
        help_text=_(
            "The name of the NAPALM driver to use when interacting with devices"
        ),
    )
    napalm_args = models.JSONField(
        blank=True,
        null=True,
        verbose_name="NAPALM arguments",
        help_text=_(
            "Additional arguments to pass when initiating the NAPALM driver (JSON format)"
        ),
    )

    class Meta:
        ordering = ("pk",)

    def __str__(self):
        return f"{self.platform.name} -> {self.napalm_driver}"

    def get_absolute_url(self):
        return reverse("plugins:netbox_napalm_plugin:napalmplatformconfig", args=[self.pk])



import django_tables2 as tables
from django_tables2.utils import Accessor
from netbox.tables import ChoiceFieldColumn, NetBoxTable

from .models import NapalmPlatformConfig


class NapalmPlatformConfigTable(NetBoxTable):
    name = tables.Column(
        accessor=Accessor('platform__name'),
        linkify={
            'viewname': 'plugins:netbox_napalm_plugin:napalmplatformconfig',
            'args': [Accessor('pk')],
        }
    )

    class Meta(NetBoxTable.Meta):
        model = NapalmPlatformConfig
        fields = ("pk", "name", "napalm_driver", "napalm_args", "actions")
        default_columns = ("name", "napalm_driver")



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

plugin_buttons = [
    PluginMenuButton(
        link="plugins:netbox_napalm_plugin:napalmplatformconfig_add",
        title="Add",
        icon_class="mdi mdi-plus-thick",
        color=ButtonColorChoices.GREEN,
    )
]

menu_items = (
    PluginMenuItem(
        link="plugins:netbox_napalm_plugin:napalmplatformconfig_list",
        link_text="Napalm",
        buttons=plugin_buttons,
    ),
)



from dcim.models import Device
from django.shortcuts import get_object_or_404, redirect, render
from extras.plugins.utils import get_plugin_config
from netbox.api.exceptions import ServiceUnavailable
from netbox.api.pagination import StripCountAnnotationsPaginator
from netbox.api.viewsets import NetBoxModelViewSet

from rest_framework.decorators import action
from rest_framework.response import Response

from netbox_napalm_plugin import filtersets
from netbox_napalm_plugin.models import NapalmPlatformConfig

from . import serializers


class NapalmPlatformConfigViewSet(NetBoxModelViewSet):
    queryset = NapalmPlatformConfig.objects.prefetch_related(
        "platform",
        "tags",
    )
    serializer_class = serializers.NapalmPlatformConfigSerializer
    filterset_class = filtersets.NapalmPlatformConfigFilterSet
    pagination_class = StripCountAnnotationsPaginator

    @action(detail=True, url_path="napalm")
    def napalm(self, request, pk):
        """
        Execute a NAPALM method on a Device
        """
        device = get_object_or_404(Device.objects.all(), pk=pk)
        if not device.primary_ip:
            raise ServiceUnavailable(
                "This device does not have a primary IP address configured."
            )
        if device.platform is None:
            raise ServiceUnavailable("No platform is configured for this device.")
        # Checks to see if NapalmPlatform object exists
        if not NapalmPlatformConfig.objects.filter(platform=device.platform).exists():
            raise ServiceUnavailable(
                f"No NAPALM Platform Mapping is configured for this device's platform: {device.platform}."
            )
        if NapalmPlatformConfig.objects.get(platform=device.platform).napalm_driver == "":
            raise ServiceUnavailable(
                f"No NAPALM driver is configured for this device's platform: {device.platform}."
            )

        # Check for primary IP address from NetBox object
        if device.primary_ip:
            host = str(device.primary_ip.address.ip)
        else:
            # Raise exception for no IP address and no Name if device.name does not exist
            if not device.name:
                raise ServiceUnavailable(
                    "This device does not have a primary IP address or device name to lookup configured."
                )
            try:
                # Attempt to complete a DNS name resolution if no primary_ip is set
                host = socket.gethostbyname(device.name)
            except socket.gaierror:
                # Name lookup failure
                raise ServiceUnavailable(
                    f"Name lookup failure, unable to resolve IP address for {device.name}. Please set Primary IP or "
                    f"setup name resolution."
                )

        # Check that NAPALM is installed
        try:
            import napalm
            from napalm.base.exceptions import ModuleImportError
        except ModuleNotFoundError as e:
            if getattr(e, "name") == "napalm":
                raise ServiceUnavailable(
                    "NAPALM is not installed. Please see the documentation for instructions."
                )
            raise e

        # Validate the configured driver
        try:
            driver = napalm.get_network_driver(NapalmPlatformConfig.objects.get(platform=device.platform).napalm_driver)
        except ModuleImportError:
            raise ServiceUnavailable(
                "NAPALM driver for platform {} not found: {}.".format(
                    device.platform, NapalmPlatformConfig.objects.get(platform=device.platform).napalm_driver
                )
            )

        # Verify user permission
        if not request.user.has_perm("dcim.napalm_read_device"):
            return HttpResponseForbidden()

        napalm_methods = request.GET.getlist("method")
        response = {m: None for m in napalm_methods}

        username = get_plugin_config('netbox_napalm_plugin', 'NAPALM_USERNAME')
        password = get_plugin_config('netbox_napalm_plugin', 'NAPALM_PASSWORD')
        timeout = get_plugin_config('netbox_napalm_plugin', 'NAPALM_TIMEOUT')
        optional_args = get_plugin_config('netbox_napalm_plugin', 'NAPALM_ARGS').copy()
        if NapalmPlatformConfig.objects.get(platform=device.platform).napalm_args is not None:
            optional_args.update(NapalmPlatformConfig.objects.get(platform=device.platform).napalm_args)
        # Update NAPALM parameters according to the request headers
        for header in request.headers:
            if header[:9].lower() != "x-napalm-":
                continue

            key = header[9:]
            if key.lower() == "username":
                username = request.headers[header]
            elif key.lower() == "password":
                password = request.headers[header]
            elif key:
                optional_args[key.lower()] = request.headers[header]

        # Connect to the device
        d = driver(
            hostname=host,
            username=username,
            password=password,
            timeout=timeout,
            optional_args=optional_args,
        )
        try:
            d.open()
        except Exception as e:
            raise ServiceUnavailable(
                "Error connecting to the device at {}: {}".format(host, e)
            )

        # Validate and execute each specified NAPALM method
        for method in napalm_methods:
            if not hasattr(driver, method):
                response[method] = {"error": "Unknown NAPALM method"}
                continue
            if not method.startswith("get_"):
                response[method] = {"error": "Only get_* NAPALM methods are supported"}
                continue
            try:
                response[method] = getattr(d, method)()
            except NotImplementedError:
                response[method] = {
                    "error": "Method {} not implemented for NAPALM driver {}".format(
                        method, driver
                    )
                }
            except Exception as e:
                response[method] = {"error": "Method {} failed: {}".format(method, e)}
        d.close()

        return Response(response)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = ['napalm<5.0']

setup(
    author="Arthur Hanson",
    author_email='ahanson@netboxlabs.com',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="NetBox plugin for Napalm.",
    install_requires=requirements,
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='netbox_napalm_plugin',
    name='netbox_napalm_plugin',
    packages=find_packages(include=['netbox_napalm_plugin', 'netbox_napalm_plugin.*']),
    test_suite='tests',
    url='https://github.com/netbox-community/netbox-napalm',
    version='0.1.5',
    zip_safe=False,
)



# Generated by Django 4.1.5 on 2023-02-15 17:52

from django.db import migrations


def forwards_migrate_napalm(apps, schema_editor):
    Platform = apps.get_model("dcim", "Platform")
    NapalmPlatformConfig = apps.get_model("netbox_napalm_plugin", "NapalmPlatformConfig")
    qs = Platform.objects.all().exclude(napalm_driver__exact="")
    for platform in qs:
        NapalmPlatformConfig.objects.create(
            platform=platform,
            napalm_driver=platform.napalm_driver,
            napalm_args=platform.napalm_args,
        )


def reverse_migrate_napalm(apps, schema_editor):
    Platform = apps.get_model("dcim", "Platform")
    NapalmPlatformConfig = apps.get_model("netbox_napalm_plugin", "NapalmPlatformConfig")
    qs = Platform.objects.all().exclude(napalm_driver__exact="")
    for platform in qs:
        NapalmPlatformConfig.objects.delete(
            platform=platform,
        )


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_napalm_plugin", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(forwards_migrate_napalm, reverse_migrate_napalm),
    ]



from dcim.constants import NONCONNECTABLE_IFACE_TYPES
from dcim.models import Device
from django.db.models import Count
from django.utils.translation import gettext as _
from netbox.views import generic
from utilities.views import ViewTab, register_model_view

from . import filtersets, forms, models, tables


class NapalmPlatformConfigView(generic.ObjectView):
    queryset = models.NapalmPlatformConfig.objects.all()


class NapalmPlatformConfigListView(generic.ObjectListView):
    queryset = models.NapalmPlatformConfig.objects.all()
    table = tables.NapalmPlatformConfigTable


class NapalmPlatformConfigEditView(generic.ObjectEditView):
    queryset = models.NapalmPlatformConfig.objects.all()
    form = forms.NapalmPlatformConfigForm


class NapalmPlatformConfigDeleteView(generic.ObjectDeleteView):
    queryset = models.NapalmPlatformConfig.objects.all()


class NAPALMViewTab(ViewTab):
    def render(self, instance):
        # Display NAPALM tabs only for devices which meet certain requirements
        if not (
            hasattr(instance.platform, "napalm")
            and instance.status == "active"
            and instance.primary_ip
            and instance.platform
            and instance.platform.napalm.napalm_driver
        ):
            return None
        return super().render(instance)


@register_model_view(Device, "status")
class DeviceStatusView(generic.ObjectView):
    additional_permissions = ["dcim.napalm_read_device"]
    queryset = Device.objects.all()
    template_name = "netbox_napalm_plugin/status.html"
    tab = NAPALMViewTab(
        label=_("Status"), permission="dcim.napalm_read_device", weight=3000
    )


@register_model_view(Device, "lldp_neighbors", path="lldp-neighbors")
class DeviceLLDPNeighborsView(generic.ObjectView):
    additional_permissions = ["dcim.napalm_read_device"]
    queryset = Device.objects.all()
    template_name = "netbox_napalm_plugin/lldp_neighbors.html"
    tab = NAPALMViewTab(
        label=_("LLDP Neighbors"), permission="dcim.napalm_read_device", weight=3100
    )

    def get_extra_context(self, request, instance):
        interfaces = (
            instance.vc_interfaces()
            .restrict(request.user, "view")
            .prefetch_related("_path")
            .exclude(type__in=NONCONNECTABLE_IFACE_TYPES)
        )

        return {
            "interfaces": interfaces,
        }


@register_model_view(Device, "config")
class DeviceConfigView(generic.ObjectView):
    additional_permissions = ["dcim.napalm_read_device"]
    queryset = Device.objects.all()
    template_name = "netbox_napalm_plugin/config.html"
    tab = NAPALMViewTab(
        label=_("Config"), permission="dcim.napalm_read_device", weight=3200
    )
